@@ -1,8 +1,8 @@
-package Blockchain::Ethereum::ABI;
-
 use v5.26;
-use strict;
-use warnings;
+use Object::Pad;
+
+package Blockchain::Ethereum::ABI 0.010;
+class Blockchain::Ethereum::ABI;
 
 =head1 NAME
 
@@ -10,11 +10,11 @@ Blockchain::Ethereum::ABI - Contract ABI utilities
 
 =head1 VERSION
 
-Version 0.009
+Version 0.010
 
 =cut
 
-our $VERSION = '0.009';
+our $VERSION = '0.010';
 
 =head1 SYNOPSIS
 
@@ -44,25 +44,6 @@ You can find documentation for this module with the perldoc command.
 
     perldoc Blockchain::Ethereum::ABI
 
-
-You can also look for information at:
-
-=over 4
-
-=item * RT: CPAN's request tracker (report bugs here)
-
-L<https://rt.cpan.org/NoAuth/Bugs.html?Dist=REFECO-Blockchain-Ethereum-ABI>
-
-=item * CPAN Ratings
-
-L<https://cpanratings.perl.org/d/REFECO-Blockchain-Ethereum-ABI>
-
-=item * Search CPAN
-
-L<https://metacpan.org/release/REFECO-Blockchain-Ethereum-ABI>
-
-=back
-
 =head1 LICENSE AND COPYRIGHT
 
 This software is Copyright (c) 2022 by REFECO.
