@@ -1,33 +1,34 @@
-package Blockchain::Ethereum::ABI::Type::String;
-
 use v5.26;
-use strict;
-use warnings;
+use Object::Pad;
 
-use parent qw(Blockchain::Ethereum::ABI::Type);
+class Blockchain::Ethereum::ABI::Type::String :isa(Blockchain::Ethereum::ABI::Type) :does(Blockchain::Ethereum::ABI::TypeRole) {
 
-sub encode {
-    my $self = shift;
-    return $self->_encoded if $self->_encoded;
+    method _configure { return }
 
-    my $hex = unpack("H*", $self->_data);
+    method encode {
 
-    # for dynamic length basic types the length must be included
-    $self->_push_dynamic($self->_encode_length(length(pack("H*", $hex))));
-    $self->_push_dynamic($self->pad_right($hex));
+        return $self->_encoded if $self->_encoded;
 
-    return $self->_encoded;
-}
+        my $hex = unpack("H*", $self->data);
 
-sub decode {
-    my $self = shift;
-    my @data = $self->_data->@*;
+        # for dynamic length basic types the length must be included
+        $self->_push_dynamic($self->_encode_length(length(pack("H*", $hex))));
+        $self->_push_dynamic($self->pad_right($hex));
 
-    my $size          = hex shift @data;
-    my $string_data   = join('', @data);
-    my $packed_string = pack("H*", $string_data);
-    return substr($packed_string, 0, $size);
-}
+        return $self->_encoded;
+    }
+
+    method decode {
+
+        my @data = $self->data->@*;
+
+        my $size          = hex shift @data;
+        my $string_data   = join('', @data);
+        my $packed_string = pack("H*", $string_data);
+        return substr($packed_string, 0, $size);
+    }
+
+};
 
 1;
 
